@@ -51,6 +51,90 @@ test_that("Prediction for classification is factor with correct levels", {
   expect_equal(levels(ti.class.formula$prediction), levels(iris$Species))
 })
 
+test_that("Prediction for classification is same as class prediction", {
+  dat <- iris[sample(nrow(iris)), ]
+  rf <- ranger(dependent.variable.name = "Species", data = dat, num.trees = 1, 
+               replace = FALSE, sample.fraction = 1)
+  pred_class <- predict(rf, dat)$predictions
+  nodes <- predict(rf, dat, type = "terminalNodes")$predictions[, 1]
+  ti <- treeInfo(rf, 1)
+  pred_ti <- sapply(nodes, function(x) {
+    ti[ti$nodeID == x, "prediction"]
+  })
+  expect_equal(pred_ti, pred_class)
+})
+
+test_that("Prediction for classification is same as class prediction, new factor", {
+  dat <- iris[sample(nrow(iris)), ]
+  dat$Species <- factor(dat$Species, levels = sample(levels(dat$Species)))
+  rf <- ranger(dependent.variable.name = "Species", data = dat, num.trees = 1, 
+               replace = FALSE, sample.fraction = 1)
+  pred_class <- predict(rf, dat)$predictions
+  nodes <- predict(rf, dat, type = "terminalNodes")$predictions[, 1]
+  ti <- treeInfo(rf, 1)
+  pred_ti <- sapply(nodes, function(x) {
+    ti[ti$nodeID == x, "prediction"]
+  })
+  expect_equal(pred_ti, pred_class)
+})
+
+test_that("Prediction for classification is same as class prediction, unused factor levels", {
+  dat <- iris[c(101:150, 51:100), ]
+  expect_warning(rf <- ranger(dependent.variable.name = "Species", data = dat, num.trees = 1, 
+    replace = FALSE, sample.fraction = 1))
+  pred_class <- predict(rf, dat)$predictions
+  nodes <- predict(rf, dat, type = "terminalNodes")$predictions[, 1]
+  ti <- treeInfo(rf, 1)
+  pred_ti <- sapply(nodes, function(x) {
+    ti[ti$nodeID == x, "prediction"]
+  })
+  expect_equal(pred_ti, pred_class)
+})
+
+test_that("Prediction for probability is same as probability prediction", {
+  dat <- iris[sample(nrow(iris)), ]
+  rf <- ranger(dependent.variable.name = "Species", data = dat, num.trees = 1, 
+               sample.fraction = 1, replace = FALSE, probability = TRUE)
+  ti <- treeInfo(rf)
+  pred_prob <- predict(rf, dat)$predictions
+  nodes <- predict(rf, dat, type = "terminalNodes")$predictions[, 1]
+  pred_ti <- t(sapply(nodes, function(x) {
+    as.matrix(ti[ti$nodeID == x, 8:10])
+  }))
+  colnames(pred_ti) <- gsub("pred\\.", "", colnames(ti)[8:10])
+  expect_equal(pred_prob, pred_ti)
+})
+
+test_that("Prediction for probability is same as probability prediction, new factor", {
+  dat <- iris[sample(nrow(iris)), ]
+  dat$Species <- factor(dat$Species, levels = sample(levels(dat$Species)))
+  rf <- ranger(dependent.variable.name = "Species", data = dat, num.trees = 1, 
+               sample.fraction = 1, replace = FALSE, probability = TRUE)
+  ti <- treeInfo(rf)
+  pred_prob <- predict(rf, dat)$predictions
+  nodes <- predict(rf, dat, type = "terminalNodes")$predictions[, 1]
+  pred_ti <- t(sapply(nodes, function(x) {
+    as.matrix(ti[ti$nodeID == x, 8:10])
+  }))
+  colnames(pred_ti) <- gsub("pred\\.", "", colnames(ti)[8:10])
+  expect_equal(pred_prob, pred_ti)
+})
+
+test_that("Prediction for probability is same as probability prediction, unused factor levels", {
+  dat <- iris[c(101:150, 51:100), ]
+  dat$Species <- factor(dat$Species, levels = sample(levels(dat$Species)))
+  expect_warning(rf <- ranger(dependent.variable.name = "Species", data = dat, num.trees = 1, 
+    sample.fraction = 1, replace = FALSE, probability = TRUE))
+  ti <- treeInfo(rf)
+  pred_prob <- predict(rf, dat)$predictions
+  nodes <- predict(rf, dat, type = "terminalNodes")$predictions[, 1]
+  pred_ti <- t(sapply(nodes, function(x) {
+    as.matrix(ti[ti$nodeID == x, 8:9])
+  }))
+  colnames(pred_ti) <- gsub("pred\\.", "", colnames(ti)[8:9])
+  expect_equal(pred_prob, pred_ti)
+})
+
 test_that("Prediction for matrix classification is integer with correct values", {
   rf <- ranger(dependent.variable.name = "Species", data = data.matrix(iris), 
                num.trees = 5, classification = TRUE)
@@ -164,6 +248,23 @@ test_that("Prediction for probability is one probability per class, sum to 1", {
   expect_true(all(!ti.prob.formula$terminal | rowSums(ti.prob.formula[, 8:10]) == 1))
 })
 
+test_that("Prediction for probability has correct factor levels", {
+  dat <- iris[c(101:150, 1:100), ]
+  rf <- ranger(dependent.variable.name = "Species", data = dat, num.trees = 5, probability = TRUE)
+  
+  # Predict
+  pred_rf <- predict(rf, dat, num.trees = 1)$predictions
+  
+  # Predict with treeInfo
+  ti <- treeInfo(rf)
+  terminal_nodes <- predict(rf, dat, type = "terminalNodes")$predictions[, 1]
+  pred_ti <- as.matrix(ti[terminal_nodes + 1, grep("pred", colnames(ti))])
+  colnames(pred_ti) <- gsub("pred\\.", "", colnames(pred_ti))
+  rownames(pred_ti) <- NULL
+  
+  expect_equal(pred_rf, pred_ti)
+})
+
 ## Survival
 rf.surv.formula <- ranger(Surv(time, status) ~ ., veteran, num.trees = 5)
 rf.surv.first <- ranger(dependent.variable.name = "time", status.variable.name = "status", data = veteran[, c(3:4, 1:2, 5:8)], num.trees = 5)
@@ -234,4 +335,19 @@ test_that("Spitting value is numeric for order splitting", {
   expect_is(ti.order$splitval[!ti.order$terminal & ti.order$splitvarName == "Species"], "numeric")
 })
 
+test_that("treeInfo works for 31 unordered factor levels but not for 32", {
+  n <- 31
+  dt <- data.frame(x = factor(1:n, ordered = FALSE),  
+                   y = rbinom(n, 1, 0.5))
+  rf <- ranger(y ~ ., data = dt, num.trees = 10, splitrule = "extratrees")
+  expect_silent(treeInfo(rf))
+  
+  n <- 32
+  dt <- data.frame(x = factor(1:n, ordered = FALSE),  
+                   y = rbinom(n, 1, 0.5))
+  rf <- ranger(y ~ ., data = dt, num.trees = 10, splitrule = "extratrees")
+  expect_warning(treeInfo(rf), "Unordered splitting levels can only be shown for up to 31 levels.")
+})
+
+
 
